﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Payment.Transfer.Form', {
    extend: 'System.Control.EntityForm',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Payment.Transfer.Form',
    autoScroll: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 800,
            large: 0
        }
    },
    viewModel: {
        data: {
            owner: {},
            Type:null,
            Amount: 0,
            PaymentKvid: null,
            AccountName:'',
            AccountKvid: null,
            Remark: '',
            ButtonText: null,
            ButtonIcon: null
        },
        formulas: {
            amountChineseBig: function(get) {
                let _amount = get('Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            }
        }
    },
    tbar: [{
        margin: '0 3 -10 3',
        xtype: 'displayfield',
        bind:{
        	value: '<p style="color:red">当前要充值的到账类型为:{Type}</p>'
        },
        fieldStyle: {
            fontSize: '18px'
        }
    }, '->', {
        iconAlign: 'left',
        handler: '_onSaveEntity',
        bind: {
            text: '{ButtonText}',
            iconCls: '{ButtonIcon}',
            disabled: '{!formIsValid}',
            hidden: '{!ButtonText}'
        }
    }],
    defaults: {
        margin: '0 10 10 10',
        labelWidth: 60,
        labelAlign: 'right',
        xtype: 'textfield'
    },
    items: [{
    	xtype:'textfield',
    	fieldLabel: '充值账户',
        editable: false,
        allowBlank: false,
        reference: 'account',
        responsiveCls: 'big-100 small-100',
        triggers: {
		    _selector: {
		        cls: 'x-fa fa-ellipsis-h',
		        hideOnReadOnly: true,
		        handler:'_onSelectAccount'
		    }
		},
        bind: {
            value: '{AccountName}'
        }
    }, {
        xtype: 'numberfield',
        fieldLabel: '充值金额',
        emptyText: '请输入数字',
        hideTrigger: true,
        responsiveCls: 'big-100 small-100',
        reference: 'amount',
        minValue: 0,
        decimalPrecision: 2,
        bind: {
            value: '{Amount}'
        },
        allowBlank: false
    }, {
        xtype: 'textfield',
        responsiveCls: 'big-100 small-100',
        fieldLabel: '金额大写',
        readOnly: true,
        bind: {
            value: '{amountChineseBig}'
        }
    }, {
        xtype: 'textarea',
        fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
        responsiveCls: 'big-100 small-100',
        bind: '{Remark}'
    }],
    setOwner:function(_entity,_ctrl){
    	let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        _vm.set('ButtonText', _ctrl.text);
        _vm.set('ButtonIcon', _ctrl.iconCls);
        let _amount = parseFloat(parseFloat(_entity.Amount).toFixed(2));
        _refs.amount.setMaxValue(_amount);
        _vm.set('Amount', _amount);
        _vm.set('PaymentKvid', _entity.PaymentKvid);
        _vm.set('AccountKvid', _entity.AccountKvid);
        _vm.set('AccountName', _entity.AccountName);
        if(_entity.AccountKvid)_refs.account.setReadOnly(true);
        _vm.set('owner', _entity);
        _vm.set('Type','储值充值');
        if(_entity.Type==="UnBiz")_vm.set('Type','非业务');
        if(_entity.Type==="Refund")_vm.set('Type','退款');
    },
    // setOwner2: function(_entity, _ctrl) {
    //     let _view = this;
    //     let _vm = _view.getViewModel();
    //     let _refs = _view.getReferences();
    //     _vm.set('ButtonText', _ctrl.text);
    //     _vm.set('ButtonIcon', _ctrl.iconCls);
    //     let _amount = parseFloat(parseFloat(_entity.Amount - _entity.AmountUsed).toFixed(2));
    //     _refs.amount.setMaxValue(_amount);
    //     _vm.set('Amount', _amount);
    //     _vm.set('PaymentKvid', _entity.Kvid);
    //     _vm.set('owner', _entity);
    //     _vm.set('Type','储值充值');
    //     if(_entity.Type==="UnBiz")_vm.set('Type','非业务');
    //     if(_entity.Type==="Refund")_vm.set('Type','退款');
    // },
    controller: {
    	_onSelectAccount: function (_ctrl, _trigger) {
    		let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
	        let _form = Ext.create('App.Control.Kivii.Finances.Transfer.Account.Selector');
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: '选择充值目标账户...',
                iconCls: 'x-fa fa-users',
                iconAlign: 'left',
                autoScroll: true,
                width: 700,
                height: 400,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _form.setOwner(_vm.data.owner);
            _win.show();
            _form.on('entitySelected', function(_ctrl, _entity) {
                _win.close();
                _vm.set('AccountKvid',_entity[0].Kvid);
                _vm.set('AccountName',_entity[0].Name);
            });
	    },
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            if (_vm.data.Amount <= 0) {
                Ext.toast('无可操作金额!');
                return;
            }
            let _jsonData = {};
            _jsonData.Amount = _vm.data.Amount;
            _jsonData.PaymentKvid = _vm.data.PaymentKvid;
            _jsonData.AccountKvid = _vm.data.AccountKvid;
            _jsonData.Remark = _vm.data.Remark;
            if (_jsonData === {}) return;
            _view.mask('数据通讯中...');
            let _url = '/Restful/' + _view.entityType + '/Transfer.json';
            // return;
            Ext.Ajax.request({
                url: _url,
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _view.unmask();
                    Ext.toast('操作成功!');
                    _view.fireEvent('entityTransfer', _controller, _rtns);
                },
                failure: function(_response) {
                    _view.unmask();
                    Ext.toast('操作失败!');
                }
            });
        }
    }
});