﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class BillExtension
    {
        public static BillDetail Offset(this BillDetail billDetail, string remark)
        {
            billDetail.ThrowIfNull("账单明细不能为空!");
            (billDetail.OffsetKvid != Guid.Empty).ThrowIfTrue("当前账单明细已作废!无需重复操作!");

            #region payment冲账
            var newBillDetailKvid = Guid.NewGuid();
            billDetail.OffsetKvid = newBillDetailKvid;
            billDetail.AddOnlyProperties(o => o.OffsetKvid);
            billDetail.Remark = $"{remark}";
            billDetail.AddOnlyProperties(o => o.Remark);

            //新建一个和原来一样的
            var offsetBillDetail = new BillDetail();
            offsetBillDetail.PopulateWith(billDetail);
            offsetBillDetail.OffsetKvid = billDetail.Kvid;//新的冲帐关联用原来的Kvid
            offsetBillDetail.Kvid = newBillDetailKvid;
            offsetBillDetail.SerialNumber = billDetail.SerialNumber.IsNullOrEmpty() ? "" : ("-" + billDetail.SerialNumber);
            offsetBillDetail.Remark = $"{remark}";
            offsetBillDetail.Amount = 0 - offsetBillDetail.Amount;//反向值
            offsetBillDetail.OperatorName = KiviiContext.CurrentMember.FullName;
            offsetBillDetail.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            #endregion

            return offsetBillDetail;
        }

        public static Bill Accept(this Bill bill, List<Settlement> settlements = null, IDbConnection conn = null)
        {
            bill.ThrowIfNull("账单不能为空!");
            //settlements.ThrowIfNullOrEmpty("结算不能为空!");

            var rtns = new Bill();

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Payment>();
            }
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();

            try
            {
                rtns.PopulateWith(bill);
                var billKvid = Guid.NewGuid();
                rtns.Kvid = billKvid;
                rtns.OperateTime = DateTime.Now;
                rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                rtns.OperatorName = KiviiContext.CurrentMember.FullName;
                rtns.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                rtns.OwnerName = KiviiContext.CurrentMember.DepartmentName;
                conn.Insert(rtns);

                if (!settlements.IsNullOrEmpty())
                {
                    foreach (var settlement in settlements)
                    {
                        var billDetail = new BillDetail();
                        billDetail.BillKvid = billKvid;

                        billDetail.BizId = settlement.SerialNumber;
                        billDetail.BizType = typeof(Settlement).FullName;
                        billDetail.BizKvid = settlement.Kvid;

                        billDetail.Type = settlement.Type;
                        billDetail.Amount = settlement.Amount;
                        billDetail.Currency = settlement.Currency;
                        billDetail.Summary = settlement.Summary;
                        billDetail.Remark = settlement.Remark;
                        billDetail.OperateTime = DateTime.Now;
                        billDetail.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                        billDetail.OperatorName = KiviiContext.CurrentMember.FullName;
                        conn.Insert(billDetail);
                    }
                }
                trans?.Commit();
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }


        /// <summary>
        /// 账单验证是否归档方法
        /// </summary>
        /// <param name="bill"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Bill Archiving(this Bill bill,IDbConnection conn = null)
        {
            bill.ThrowIfNull("账单不能为空!");
            
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Bill>();
            }
            var queryDetails = conn.From<BillDetail>();
            queryDetails.Where(o => o.BillKvid == bill.Kvid);
            queryDetails.Select(o => o.BizKvid);
            //var details = conn.Select(queryDetails);
            var querySettlements=conn.From<Settlement>();
            querySettlements.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Kvid, queryDetails));
            querySettlements.Select(o => new { o.Amount, o.AmountPlan, o.AmountPayment });
            var settlements = conn.Select(querySettlements);
            if (settlements.IsNullOrEmpty())//未找到结算信息 说明此账单没有费用 
            {
                bill.Amount = 0;
                bill.AddOnlyProperties(o => o.Amount);
                bill.AmountPlan = 0;
                bill.AddOnlyProperties(o => o.AmountPlan);
                bill.AmountPayment = 0;
                bill.AddOnlyProperties(o => o.AmountPayment);
                conn.UpdateOnly(bill);
                return bill;
            }
            bill.Amount = settlements.Sum(o => o.Amount);
            bill.AddOnlyProperties(o => o.Amount);
            bill.AmountPlan = settlements.Sum(o => o.AmountPlan);
            bill.AddOnlyProperties(o => o.AmountPlan);
            bill.AmountPayment = settlements.Sum(o => o.AmountPayment);
            bill.AddOnlyProperties(o => o.AmountPayment);
            conn.UpdateOnly(bill);
            return bill;
        }
    }
}
