﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "已结算未收费")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementUnPayAnalysis : RestfulExecution<Settlement>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public bool GroupByOwner { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }
        public List<string> Types { get; set; }
        public bool IsOperated { get; set; }//如果True则查询OperateTime否则默认CreateTime

        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new StatisticResponse<Analysis>();
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var sqlExpress = conn.From<Settlement>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
            if (IsOperated) sqlExpress.And(o => o.OperateTime >= beginTime && o.OperateTime < endTime);
            else sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            //sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));

            #region 如果是GroupByOwner的处理情况
            if (GroupByOwner)
            {
                sqlExpress.GroupBy(o => new { o.OwnerName, o.OwnerKvid });
                sqlExpress.Select(o => new
                {
                    OwnerKvid = o.OwnerKvid,
                    OwnerName = o.OwnerName,
                    Summary = o.OwnerName,
                    Quantity = Sql.Count(o.Kvid),
                    Amount = Sql.Sum(o.Amount),
                    AmountPayment = Sql.Sum(o.AmountPayment),
                    QuantityPayment = Sql.Sum(o.Amount == o.AmountPayment ? 1 : 0),
                    AmountUnpaid = Sql.Sum(o.Amount - o.AmountPayment),
                    QuantityUnpaid = Sql.Sum(o.AmountPayment < o.Amount ? 1 : 0)
                });
                rtns.Results = conn.Select<Analysis>(sqlExpress);
                rtns.BeginTime = BeginTime;
                rtns.EndTime = EndTime;
                if (rtns.Results.IsNullOrEmpty()) return rtns;
                rtns.TotalQuantity = rtns.Results.Sum(o => o.Quantity);
                rtns.TotalAmount = rtns.Results.Sum(o => o.Amount);
                rtns.TotalAmountPayment = rtns.Results.Sum(o => o.AmountPayment);
                rtns.TotalQuantityPayment = rtns.Results.Sum(o => o.QuantityPayment);
                rtns.TotalAmountUnpaid = rtns.Results.Sum(o => o.AmountUnpaid);
                rtns.TotalQuantityUnpaid = rtns.Results.Sum(o => o.QuantityUnpaid);

                rtns.Total = rtns.Results.Count;
                return rtns;
            }
            #endregion
            //以下是非GroupByOwner的情况
            sqlExpress.GroupBy(o => o.Type);
            sqlExpress.Select(o => new
            {
                Summary = o.Type,
                Quantity = Sql.Count(o.Kvid),
                Amount = Sql.Sum(o.Amount),
                AmountPayment = Sql.Sum(o.AmountPayment),
                QuantityPayment = Sql.Sum(o.Amount == o.AmountPayment ? 1 : 0),
                AmountUnpaid = Sql.Sum(o.Amount - o.AmountPayment),
                QuantityUnpaid = Sql.Sum(o.AmountPayment < o.Amount ? 1 : 0)
            });
            var results = conn.Select<Analysis>(sqlExpress);
            rtns.Results = new List<Analysis>();
            if (results.IsNullOrEmpty()) return rtns;
            if (!Types.IsNullOrEmpty())
            {
                rtns.Results.AddRange(results.Where(o => Types.Contains(o.Summary)));
                results.RemoveAll(o => Types.Contains(o.Summary));

                //待收费其他
                var rtnOthers = new Analysis();
                rtnOthers.Summary = "Other";
                rtnOthers.BeginTime = BeginTime;
                rtnOthers.EndTime = EndTime;
                rtnOthers.Quantity = results.Count();
                rtnOthers.Amount = results.Sum(o => o.Amount);
                rtnOthers.AmountPayment = results.Sum(o => o.AmountPayment);
                rtnOthers.QuantityPayment = results.Count(o => o.AmountPayment == o.Amount);
                rtnOthers.AmountUnpaid = results.Sum(o => o.Amount - o.AmountPayment);
                rtnOthers.QuantityUnpaid = results.Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
                rtns.Results.Add(rtnOthers);
            }
            else rtns.Results.AddRange(results);

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
}
