﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    public class StatisticResponse<T> : RestfulQueryResponse<T>
    {
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }
        public string OwnerName { get; set; }

        public Guid OwnerKvid { get; set; }

        public string Description { get; set; }
        public decimal TotalAmount { get; set; }
        public decimal TotalAmountSplited { get; set; }
        public decimal TotalAmountNotSplited { get; set; }
        public decimal TotalAmountInvoice { get; set; }
        public decimal TotalAmountNotInvoice { get; set; }
        public decimal TotalAmountUsed { get; set; }
        public decimal TotalAmountNotUsed { get; set; }
        public decimal TotalAmountPayment { get; set; }
        public decimal TotalAmountUnsettled { get; set; }
        public decimal TotalAmountUnpaid { get; set; }
        public decimal TotalAmountOffset { get; set; }
        public decimal TotalAmountRefund { get; set; }
        public decimal TotalAmountUnBiz { get; set; }

        public decimal TotalQuantity { get; set; }
        public decimal TotalQuantitySplited { get; set; }
        public decimal TotalQuantityNotSplited { get; set; }
        public decimal TotalQuantityInvoice { get; set; }
        public decimal TotalQuantityNotInvoice { get; set; }
        public decimal TotalQuantityUsed { get; set; }
        public decimal TotalQuantityNotUsed { get; set; }
        public decimal TotalQuantityPayment { get; set; }
        public decimal TotalQuantityUnsettled { get; set; }
        public decimal TotalQuantityUnpaid { get; set; }
        public decimal TotalQuantityOffset { get; set; }
        public decimal TotalQuantityRefund { get; set; }
        public decimal TotalQuantityUnBiz { get; set; }

    }

    //分析维度
    public enum AnalysisType
    {
        Depart = 0,//部门
        Month,//月份
        Type,//类型
        Payer//付款单位
    }

    #region 明细统计
    [Api(Description = "发票统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceStatistic : RestfulExecution<Invoice>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool Offseted { get; set; }

        /// <summary>
        /// 用于查询借票是否到账使用 
        /// </summary>
        public bool? Payed { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Invoice>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.Type != "Relation");
            if (Offseted)//只查作废或者红字的发票
            {
                sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime && o.Amount < 0);
            }
            else
            {
                //查当月的全部发票(包含当月到账的借票)
                sqlExpress.And(o => ((o.OperateTime >= BeginTime && o.OperateTime < EndTime) || (o.PayedTime.Value >= BeginTime && o.PayedTime.Value < EndTime && o.OperateTime < BeginTime)));
                if (Payed != null && Payed.Value) sqlExpress.And(o => o.AmountPayment == o.Amount);
                if (Payed != null && !Payed.Value) sqlExpress.And(o => o.AmountPayment < o.Amount);
            }
            var results = autoQuery.Execute(Request, conn, request, sqlExpress);

            var rtns = new StatisticResponse<Invoice>();
            rtns.PopulateWith(results);
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            rtns.TotalAmount = rtns.Results.Sum(o => o.Amount);
            rtns.TotalAmountPayment = rtns.Results.Sum(o => o.AmountPayment);
            return rtns;
        }
    }

    [Api(Description = "结算统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementStatistic : RestfulExecution<Settlement>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool? Payed { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Settlement>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            //sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
            if (Payed != null)
            {
                if (Payed.Value) sqlExpress.And(o => o.Amount == o.AmountPayment);
                else sqlExpress.And(o => o.Amount > o.AmountPayment);
            }
            sqlExpress.And(o => o.CreateTime >= BeginTime && o.CreateTime < EndTime);

            var results = autoQuery.Execute(Request, conn, request, sqlExpress);

            var rtns = new StatisticResponse<Settlement>();
            rtns.PopulateWith(results);
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            rtns.TotalAmount = rtns.Results.Where(o => o.Amount >= 0).Sum(o => o.Amount);
            rtns.TotalQuantity = rtns.Results.Where(o => o.Amount >= 0).Count();
            rtns.TotalAmountPayment = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment > 0).Sum(o => o.AmountPayment);
            rtns.TotalQuantityPayment = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment == o.Amount).Count();
            rtns.TotalAmountUnpaid = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment < o.Amount).Sum(o => o.Amount - o.AmountPayment);
            rtns.TotalQuantityUnpaid = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment == 0).Count();
            rtns.TotalAmountOffset = rtns.Results.Where(o => o.Amount < 0).Sum(o => o.Amount);
            rtns.TotalQuantityOffset = rtns.Results.Where(o => o.Amount < 0).Count();
            return rtns;
        }
    }

    [Api(Description = "到账统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentStatistic : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public bool? IsPayee { get; set; }//查收款还是付款

        public bool? Used { get; set; }//到账是否已被使用

        public bool? Invoiced { get; set; }//到账是否开票

        public bool Limited { get; set; } = true;

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            if (dynamicParams.ContainsKey("OwnerKvid")) dynamicParams.Remove("OwnerKvid");
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
            if (OwnerKvids.IsNullOrEmpty())//没传部门信息就查全部到账的数据
            {
                sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Bank));
            }
            else//指定了部门就只查部门认领的到账数据内容
            {
                sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split, PaymentType.Refund, PaymentType.UnBiz));
                sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            }
            //if (OwnerType == OwnerType.Organization) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid || o.OwnerKvid == Guid.Empty);
            //if (OwnerType == OwnerType.Department) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.DepartmentKvid || o.OwnerKvid == Guid.Empty);
            //if (OwnerType == OwnerType.Member) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid || o.OwnerKvid == Guid.Empty);
            if (IsPayee != null)
            {
                var queryAccount = conn.From<Account>();
                queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
                queryAccount.Select(o => o.Kvid);

                if (IsPayee.Value) sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));//收款
                else sqlExpress.And(o => Sql.In(o.PayeeAccountKvid, queryAccount));//付款
            }
            if (Used != null)
            {
                if (Used.Value) sqlExpress.And(o => o.AmountUsed == o.Amount);
                else sqlExpress.And(o => o.AmountUsed < o.Amount);
            }
            if (Invoiced != null)
            {
                if (Invoiced.Value) sqlExpress.And(o => o.AmountInvoice == o.Amount);
                else sqlExpress.And(o => o.AmountInvoice < o.Amount);
            }
            if (!Limited) sqlExpress.Rows = null;
            var results = autoQuery.Execute(Request, conn, request, sqlExpress);
            var rtns = new StatisticResponse<Payment>();
            rtns.PopulateWith(results);
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            rtns.TotalAmount = rtns.Results.Where(o => o.Amount >= 0).Sum(o => o.Amount);
            rtns.TotalQuantity = rtns.Results.Where(o => o.Amount >= 0).Count();
            rtns.TotalAmountSplited = rtns.Results.Where(o => o.AmountSplited > 0).Sum(o => o.AmountSplited);
            rtns.TotalQuantitySplited = rtns.Results.Where(o => o.Amount >= 0 && o.AmountSplited == o.Amount).Count();
            rtns.TotalAmountInvoice = rtns.Results.Where(o => o.Amount >= 0 && o.AmountInvoice > 0).Sum(o => o.AmountInvoice);
            rtns.TotalQuantityInvoice = rtns.Results.Where(o => o.Amount >= 0 && o.AmountInvoice == o.Amount).Count();
            rtns.TotalAmountUsed = rtns.Results.Where(o => o.Amount >= 0 && o.AmountUsed > 0).Sum(o => o.AmountUsed);
            rtns.TotalQuantityUsed = rtns.Results.Where(o => o.Amount >= 0 && o.AmountUsed == o.Amount).Count();
            return rtns;
        }
    }

    [Api(Description = "账户统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountStatistic : RestfulExecution<Account>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            var conn = KiviiContext.GetOpenedDbConnection<Account>();
            var queryKvids = conn.From<Account>();
            queryKvids.Where(o => o.Type == AccountType.Deposit);
            if (!OwnerKvids.IsNullOrEmpty()) queryKvids.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            queryKvids.Select(o => o.Kvid);
            var sqlExpress = conn.From<Account>();
            sqlExpress.Where(o => o.Type == AccountType.Deposit);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            var results = conn.Select(sqlExpress);
            var rtns = new StatisticResponse<Analysis>();
            rtns.Results = new List<Analysis>();
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            var queryDetails = conn.From<AccountDetail>();
            queryDetails.Where(o => Sql.In(o.AccountKvid, queryKvids));
            queryDetails.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            var allDetails = conn.Select(queryDetails);

            if (!OwnerKvids.IsNullOrEmpty())
            {
                foreach (var item in results)
                {
                    var details = allDetails.Where(o => o.AccountKvid == item.Kvid).ToList();
                    var rtn = new Analysis();
                    rtn.Summary = item.Name;
                    rtn.Amount = item.Amount;
                    rtn.AmountPayment = details.Where(o => o.PayeeAccountKvid == item.Kvid).Sum(o => o.AmountPayment);
                    rtn.AmountUsed = details.Where(o => o.PayerAccountKvid == item.Kvid).Sum(o => o.AmountPayment);
                    rtns.Results.Add(rtn);
                }
            }
            else
            {
                var group = results.GroupBy(o => o.OwnerKvid);
                foreach (var kv in group)
                {
                    var details = allDetails.Where(o => kv.ToList().ConvertAll(p => p.Kvid).Contains(o.AccountKvid)).ToList();
                    var rtn = new Analysis();
                    rtn.Summary = kv.FirstOrDefault().OwnerName;
                    rtn.Quantity = kv.Count();
                    rtn.Amount = kv.Sum(o => o.Amount);
                    rtn.AmountPayment = details.Where(o => kv.ToList().ConvertAll(p => p.Kvid).Contains(o.PayeeAccountKvid)).Sum(o => o.AmountPayment);
                    rtn.AmountUsed = details.Where(o => kv.ToList().ConvertAll(p => p.Kvid).Contains(o.PayerAccountKvid)).Sum(o => o.AmountPayment);
                    rtns.Results.Add(rtn);
                }
            }
            return rtns;
        }
    }
    #endregion

    #region 分析
    [Api(Description = "到账认领分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentSplitAnalysis : RestfulExecution<Payment>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }


        public AnalysisType Type { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split));
            sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            //if (OwnerKvid != Guid.Empty) sqlExpress.And(o => o.OwnerKvid == OwnerKvid);
            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            sqlExpress.OrderBy(o => o.OwnerName);
            //sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.Amount, o.AmountInvoice, o.AmountUsed, o.OperateTime, o.CreateTime, o.Type });
            //var results = conn.Select(sqlExpress);
            var rtns = new StatisticResponse<Analysis>();
            //if (results.IsNullOrEmpty()) return rtns;
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            //foreach (var item in results)
            //{
            //    item.Summary = item.OperateTime.ToString("yyyy-MM");
            //}
            if (Type == AnalysisType.Depart)
            {
                sqlExpress.GroupBy(o => o.OwnerName);
                sqlExpress.Select(o => new
                {
                    Summary = o.OwnerName,
                    Amount = Sql.Sum(o.Amount),
                    Quantity = Sql.Count(o.Kvid),
                    AmountInvoice = Sql.Sum(o.AmountInvoice),
                    QuantityInvoice = Sql.Sum(o.AmountInvoice == o.Amount ? 1 : 0),
                    AmountUsed = Sql.Sum(o.AmountUsed),
                    QuantityUsed = Sql.Sum(o.AmountUsed == o.Amount ? 1 : 0),
                    AmountSplit = Sql.Sum(o.Amount),
                    QuantitySplit = Sql.Count(o.Kvid)
                });
                rtns.Results = conn.Select<Analysis>(sqlExpress);
                //var group = results.GroupBy(o => o.OwnerName);
                //foreach (var kv in group)
                //{
                //    var rtn = new Analysis();
                //    rtn.Summary = kv.Key;
                //    rtn.Amount = kv.Sum(o => o.Amount);
                //    rtn.Quantity = kv.Count();
                //    rtn.AmountInvoice = kv.Sum(o => o.AmountInvoice);
                //    rtn.QuantityInvoice = kv.Count(o => o.Amount == o.AmountInvoice);
                //    rtn.AmountUsed = kv.Sum(o => o.AmountUsed);
                //    rtn.QuantityUsed = kv.Count(o => o.Amount == o.AmountUsed);
                //    rtn.AmountSplit = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.Amount);
                //    rtn.QuantitySplit = kv.Count(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos);
                //    rtn.AmountRefund = kv.Where(o => o.Type == PaymentType.Refund).Sum(o => o.Amount);
                //    rtn.QuantityRefund = kv.Count(o => o.Type == PaymentType.Refund);
                //    rtn.AmountUnBiz = kv.Where(o => o.Type == PaymentType.UnBiz).Sum(o => o.Amount);
                //    rtn.QuantityUnBiz = kv.Count(o => o.Type == PaymentType.UnBiz);
                //    rtns.Results.Add(rtn);
                //}
            }
            else
            {
                sqlExpress.GroupBy(o => o.OperateTime.ToString("yyyy-MM"));
                sqlExpress.Select(o => new
                {
                    Summary = o.OperateTime.ToString("yyyy-MM"),
                    Amount = Sql.Sum(o.Amount),
                    Quantity = Sql.Count(o.Kvid),
                    AmountInvoice = Sql.Sum(o.AmountInvoice),
                    QuantityInvoice = Sql.Sum(o.AmountInvoice == o.Amount ? 1 : 0),
                    AmountUsed = Sql.Sum(o.AmountUsed),
                    QuantityUsed = Sql.Sum(o.AmountUsed == o.Amount ? 1 : 0),
                    AmountSplit = Sql.Sum(o.Amount),
                    QuantitySplit = Sql.Count(o.Kvid)
                });
                rtns.Results = conn.Select<Analysis>(sqlExpress);
                //var group = results.GroupBy(o => o.Summary);
                //foreach (var kv in group)
                //{
                //    var rtn = new Analysis();
                //    rtn.Summary = kv.Key;
                //    rtn.Amount = kv.Sum(o => o.Amount);
                //    rtn.Quantity = kv.Count();
                //    rtn.AmountInvoice = kv.Sum(o => o.AmountInvoice);
                //    rtn.QuantityInvoice = kv.Count(o => o.Amount == o.AmountInvoice);
                //    rtn.AmountUsed = kv.Sum(o => o.AmountUsed);
                //    rtn.QuantityUsed = kv.Count(o => o.Amount == o.AmountUsed);
                //    rtn.AmountSplit = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.Amount);
                //    rtn.QuantitySplit = kv.Count(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos);
                //    rtn.AmountRefund = kv.Where(o => o.Type == PaymentType.Refund).Sum(o => o.Amount);
                //    rtn.QuantityRefund = kv.Count(o => o.Type == PaymentType.Refund);
                //    rtn.AmountUnBiz = kv.Where(o => o.Type == PaymentType.UnBiz).Sum(o => o.Amount);
                //    rtn.QuantityUnBiz = kv.Count(o => o.Type == PaymentType.UnBiz);
                //    rtns.Results.Add(rtn);
                //}
            }

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "到账划转分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentAssignAnalysis : RestfulExecution<Payment>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var rtns = new StatisticResponse<Analysis>();
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            //当前部门已经认领且被划转的流水Kvid
            var querySplits = conn.From<Payment>();
            querySplits.Where(o => o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Split && o.AmountSplited > 0);
            querySplits.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
            if (!OwnerKvids.IsNullOrEmpty()) querySplits.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            querySplits.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            querySplits.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Assign);
            sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
            sqlExpress.And(o => (Sql.In(o.ParentKvid, querySplits) || Sql.In(o.OwnerKvid, OwnerKvids)));
            sqlExpress.OrderBy(o => o.OwnerName);
            //sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.Amount, o.AmountInvoice, o.AmountUsed, o.OperateTime, o.CreateTime, o.Type, o.Kvid });
            
            sqlExpress.GroupBy(o => o.OwnerName);
            sqlExpress.Select(o => new
            {
                Summary = o.OwnerName,
                Amount = Sql.Sum(o.Amount),
                Quantity = Sql.Count(o.Kvid),
                AmountInvoice = Sql.Sum(o.AmountInvoice),
                QuantityInvoice = Sql.Sum(o.AmountInvoice == o.Amount ? 1 : 0),
                AmountUsed = Sql.Sum(o.AmountUsed),
                QuantityUsed = Sql.Sum(o.AmountUsed == o.Amount ? 1 : 0),
                AmountSplit = Sql.Sum(o.Amount),
                QuantitySplit = Sql.Count(o.Kvid)
            });
            rtns.Results = conn.Select<Analysis>(sqlExpress);

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    //[Api(Description = "结算分析")]
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class SettlementPayedAnalysis : RestfulExecution<Settlement>
    //{
    //    public Guid OwnerKvid { get; set; }
    //    public List<Guid> OwnerKvids { get; set; }

    //    public AnalysisType Type { get; set; } = AnalysisType.Depart;//按时间,部门,类型分析

    //    public DateTime BeginTime { get; set; }

    //    public DateTime EndTime { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
    //        if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
    //        int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
    //        if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
    //        if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

    //        BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
    //        EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
    //        if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

    //        var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
    //        var sqlExpress = conn.From<Settlement>();
    //        //sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
    //        sqlExpress.Where(o => o.CreateTime >= BeginTime && o.CreateTime < EndTime);
    //        if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
    //        //if (OwnerKvid != Guid.Empty) sqlExpress.And(o => o.OwnerKvid == OwnerKvid);
    //        sqlExpress.OrderBy(o => o.CreateTime);
    //        sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.Type, o.AmountPlan, o.Amount, o.AmountPayment, o.OperateTime, o.CreateTime });
    //        var results = conn.Select(sqlExpress);

    //        var rtns = new StatisticResponse<Settlement>();
    //        if (results.IsNullOrEmpty()) return rtns;
    //        rtns.Results = new List<Settlement>();
    //        rtns.BeginTime = BeginTime;
    //        rtns.EndTime = EndTime;
    //        foreach (var item in results)
    //        {
    //            item.Summary = item.CreateTime.ToString("yyyy-MM");
    //        }
    //        if (Type == AnalysisType.Depart)
    //        {
    //            var group = results.GroupBy(o => o.OwnerName);
    //            foreach (var kv in group)
    //            {
    //                var summary = kv.Key;
    //                if (kv.Key == "Report") summary = "报告费";
    //                if (kv.Key == "Service") summary = "服务费";
    //                if (kv.Key == "Express") summary = "快递费";
    //                if (kv.Key == "Certificate") summary = "证书费";
    //                if (kv.Key == "Tag") summary = "挂签费";
    //                if (kv.Key == "SoftwareSupervision") summary = "软件监理费";
    //                if (kv.Key == "SoftwareCostEvaluation") summary = "软件造价评估费";
    //                if (kv.Key == "Authentication") summary = "认证费";
    //                if (kv.Key == "Train") summary = "培训费";
    //                var rtn = new Settlement();
    //                rtn.Summary = summary;
    //                rtn.Amount = kv.Sum(o => o.Amount);
    //                rtn.AmountPayment = kv.Sum(o => o.AmountPayment);
    //                rtn.AmountPlan = kv.Sum(o => o.AmountPlan);
    //                rtn.Status = kv.Count();//结算数量
    //                rtn.Category = kv.Where(o => o.Amount == o.AmountPayment).Count().ToString();//已收款的结算数量
    //                rtns.Results.Add(rtn);
    //            }
    //        }
    //        else if (Type == AnalysisType.Month)
    //        {
    //            var group = results.GroupBy(o => o.Summary);
    //            foreach (var kv in group)
    //            {
    //                var summary = kv.Key;
    //                if (kv.Key == "Report") summary = "报告费";
    //                if (kv.Key == "Service") summary = "服务费";
    //                if (kv.Key == "Express") summary = "快递费";
    //                if (kv.Key == "Certificate") summary = "证书费";
    //                if (kv.Key == "Tag") summary = "挂签费";
    //                if (kv.Key == "SoftwareSupervision") summary = "软件监理费";
    //                if (kv.Key == "SoftwareCostEvaluation") summary = "软件造价评估费";
    //                if (kv.Key == "Authentication") summary = "认证费";
    //                if (kv.Key == "Train") summary = "培训费";
    //                var rtn = new Settlement();
    //                rtn.Summary = summary;
    //                rtn.Amount = kv.Sum(o => o.Amount);
    //                rtn.AmountPayment = kv.Sum(o => o.AmountPayment);
    //                rtn.AmountPlan = kv.Sum(o => o.AmountPlan);
    //                rtn.Status = kv.Count();
    //                rtn.Category = kv.Where(o => o.Amount == o.AmountPayment).Count().ToString();//已收款的结算数量
    //                rtns.Results.Add(rtn);
    //            }
    //        }
    //        else
    //        {
    //            var group = results.GroupBy(o => o.Type);
    //            foreach (var kv in group)
    //            {
    //                var summary = kv.Key;
    //                if (kv.Key == "Report") summary = "报告费";
    //                if (kv.Key == "Service") summary = "服务费";
    //                if (kv.Key == "Express") summary = "快递费";
    //                if (kv.Key == "Certificate") summary = "证书费";
    //                if (kv.Key == "Tag") summary = "挂签费";
    //                if (kv.Key == "SoftwareSupervision") summary = "软件监理费";
    //                if (kv.Key == "SoftwareCostEvaluation") summary = "软件造价评估费";
    //                if (kv.Key == "Authentication") summary = "认证费";
    //                if (kv.Key == "Train") summary = "培训费";
    //                var rtn = new Settlement();
    //                rtn.Summary = summary;
    //                rtn.Amount = kv.Sum(o => o.Amount);
    //                rtn.AmountPayment = kv.Sum(o => o.AmountPayment);
    //                rtn.AmountPlan = kv.Sum(o => o.AmountPlan);
    //                rtn.Status = kv.Count();
    //                rtn.Category = kv.Where(o => o.Amount == o.AmountPayment).Count().ToString();//已收款的结算数量
    //                rtns.Results.Add(rtn);
    //            }
    //        }

    //        return rtns;
    //    }
    //}

    //[Api(Description = "待开票流水")]
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class PaymentUnInvoiceAnalysis : RestfulExecution<Payment>
    //{
    //    public Guid OwnerKvid { get; set; }
    //    public List<Guid> OwnerKvids { get; set; }

    //    public DateTime BeginTime { get; set; }

    //    public DateTime EndTime { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
    //        if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
    //        if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

    //        BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
    //        EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
    //        if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

    //        if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
    //        if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
    //        var conn = KiviiContext.GetOpenedDbConnection<Payment>();
    //        var queryAccount = conn.From<Account>();
    //        queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
    //        queryAccount.Select(o => o.Kvid);

    //        var sqlExpress = conn.From<Payment>();
    //        if (!OwnerKvids.IsNullOrEmpty())
    //        {
    //            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Split, PaymentType.Cash, PaymentType.Pos));
    //            sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
    //        }
    //        else
    //        {
    //            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Bank, PaymentType.Cash, PaymentType.Pos));
    //        }
    //        sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
    //        //if (OwnerKvid != Guid.Empty) sqlExpress.And(o => o.OwnerKvid == OwnerKvid);
    //        //sqlExpress.And(o => o.AmountInvoice == 0 || o.AmountInvoice < o.Amount);
    //        sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
    //        sqlExpress.Select(o => new { o.AmountInvoice, o.Amount });
    //        var results = conn.Select(sqlExpress);

    //        var rtns = new StatisticResponse<Payment>();
    //        rtns.Description = "待开票流水";
    //        if (results.IsNullOrEmpty()) return rtns;
    //        rtns.Total = results.Count();
    //        rtns.TotalAmount = results.Sum(o => o.Amount);
    //        rtns.TotalAmountInvoice = results.Sum(o => o.Amount - o.AmountInvoice);
    //        rtns.TotalQuantity = results.Where(o => o.AmountInvoice == 0 || o.AmountInvoice < o.Amount).Count();

    //        //rtns.TotalAmount = results.Sum(o => o.Amount - o.AmountInvoice);
    //        //rtns.TotalQuantity = results.Count();
    //        return rtns;
    //    }
    //}

    //[Api(Description = "待使用流水")]
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class PaymentUnUsedAnalysis : RestfulExecution<Payment>
    //{
    //    public Guid OwnerKvid { get; set; }
    //    public List<Guid> OwnerKvids { get; set; }
    //    public DateTime BeginTime { get; set; }

    //    public DateTime EndTime { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
    //        if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
    //        if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

    //        BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
    //        EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
    //        if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

    //        if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
    //        if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
    //        var conn = KiviiContext.GetOpenedDbConnection<Payment>();
    //        var queryAccount = conn.From<Account>();
    //        queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
    //        queryAccount.Select(o => o.Kvid);

    //        var sqlExpress = conn.From<Payment>();
    //        sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Split, PaymentType.Cash, PaymentType.Pos));
    //        sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
    //        if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
    //        //if (OwnerKvid != Guid.Empty) sqlExpress.And(o => o.OwnerKvid == OwnerKvid);
    //        sqlExpress.And(o => o.AmountUsed == 0 || o.AmountUsed < o.Amount);
    //        sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
    //        sqlExpress.Select(o => new { o.AmountUsed, o.Amount });
    //        var results = conn.Select(sqlExpress);

    //        var rtns = new StatisticResponse<Payment>();
    //        rtns.Description = "待使用流水";
    //        if (results.IsNullOrEmpty()) return rtns;
    //        rtns.TotalAmount = results.Sum(o => o.Amount - o.AmountUsed);
    //        rtns.TotalQuantity = results.Count();
    //        return rtns;
    //    }
    //}

    [Api(Description = "已结算未收费")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementPayAnalysis : RestfulExecution<Settlement>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var sqlExpress = conn.From<Settlement>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
            sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            //sqlExpress.And(o => o.AmountPayment < o.Amount || o.AmountPayment == 0);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            //if (OwnerKvid != Guid.Empty) sqlExpress.And(o => o.OwnerKvid == OwnerKvid);
            sqlExpress.Select(o => new { o.AmountPayment, o.Amount });
            var results = conn.Select(sqlExpress);

            var rtns = new StatisticResponse<Settlement>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Total = results.Count();
            rtns.TotalAmount = results.Sum(o => o.Amount);
            rtns.TotalAmountPayment = results.Sum(o => o.AmountPayment);
            rtns.TotalQuantityPayment = results.Count(o => o.AmountPayment == o.Amount);
            rtns.TotalAmountUnpaid = results.Sum(o => o.Amount - o.AmountPayment);
            rtns.TotalQuantityUnpaid = results.Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            return rtns;
        }
    }


    [Api(Description = "发票操作分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceOperateAnalysis : RestfulExecution<Invoice>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != null) OwnerKvids.Add(OwnerKvid);
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var sqlExpress = conn.From<Invoice>();
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.Type != "Relation");
            sqlExpress.And(o => (o.OperateTime >= beginTime && o.OperateTime < endTime));
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            //if (OwnerKvid != null) sqlExpress.And(o => o.OwnerKvid == OwnerKvid.Value);
            sqlExpress.OrderBy(o => o.OwnerName);
            sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.OffsetKvid, o.Amount, o.AmountPayment, o.AmountUsed, o.OperateTime, o.CreateTime, o.Type });
            var results = conn.Select(sqlExpress);
            var rtns = new StatisticResponse<Invoice>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Total = results.Where(o => o.Amount > 0).Count();
            rtns.TotalAmount = results.Where(o => o.Amount > 0).Sum(o => o.Amount);
            rtns.TotalAmountPayment = results.Where(o => o.Amount > 0).Sum(o => o.AmountPayment);
            rtns.TotalQuantityPayment = results.Count(o => o.Amount > 0 && o.AmountPayment == o.Amount);
            rtns.TotalAmountUnpaid = results.Where(o => o.Amount > 0).Sum(o => o.Amount - o.AmountPayment);
            rtns.TotalQuantityUnpaid = results.Count(o => o.Amount > 0 && o.AmountPayment < o.Amount);
            rtns.TotalAmountOffset=results.Where(o => o.Amount <= 0).Sum(o => o.Amount);
            rtns.TotalQuantityOffset = results.Count(o => o.Amount <= 0);
            return rtns;
        }

    }

    [Api(Description = "发票分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceAnalysis : RestfulExecution<Invoice>
    {
        public Guid? OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public AnalysisType Type { get; set; } = AnalysisType.Depart;//按时间分析,否则按部门分析

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != null) OwnerKvids.Add(OwnerKvid.Value);
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var sqlExpress = conn.From<Invoice>();
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.Type != "Relation");
            sqlExpress.And(o => (o.OperateTime >= beginTime && o.OperateTime < endTime));
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            //if (OwnerKvid != null) sqlExpress.And(o => o.OwnerKvid == OwnerKvid.Value);
            sqlExpress.OrderBy(o => o.OwnerName);
            sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.OffsetKvid, o.Amount, o.AmountPayment, o.AmountUsed, o.OperateTime, o.CreateTime, o.Type });
            var results = conn.Select(sqlExpress);
            var rtns = new StatisticResponse<Analysis>();
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            foreach (var item in results)
            {
                item.Summary = item.OperateTime.ToString("yyyy-MM");
            }
            if (Type == AnalysisType.Depart)
            {
                var group = results.GroupBy(o => o.OwnerKvid);
                foreach (var kv in group)
                {
                    var rtn = new Analysis();
                    rtn.Summary = kv.FirstOrDefault().OwnerName;
                    if (kv.Key == Guid.Empty) rtn.Summary = "无部门信息发票";
                    rtn.Amount = kv.Where(o => o.Amount > 0).Sum(o => o.Amount);
                    rtn.Quantity = kv.Where(o => o.Amount > 0).Count();
                    rtn.AmountPayment = kv.Where(o => o.Amount > 0).Sum(o => o.AmountPayment);
                    rtn.QuantityPayment = kv.Where(o => o.Amount > 0 && o.AmountPayment == o.Amount).Count();
                    rtn.AmountUnpaid = kv.Where(o => o.Amount > 0).Sum(o => o.Amount - o.AmountPayment);
                    rtn.QuantityUnpaid = kv.Where(o => o.Amount > 0 && o.AmountPayment < o.Amount).Count();
                    rtn.AmountOffset = kv.Where(o => o.Amount <= 0).Sum(o => o.Amount);
                    rtn.QuantityOffset = kv.Where(o => o.Amount <= 0).Count();
                    rtns.Results.Add(rtn);
                }
            }
            else
            {
                var group = results.GroupBy(o => o.Summary);
                foreach (var kv in group)
                {
                    var rtn = new Analysis();
                    rtn.Summary = kv.Key;
                    rtn.Amount = kv.Where(o => o.Amount > 0).Sum(o => o.Amount);
                    rtn.Quantity = kv.Where(o => o.Amount > 0).Count();
                    rtn.AmountPayment = kv.Where(o => o.Amount > 0).Sum(o => o.AmountPayment);
                    rtn.QuantityPayment = kv.Where(o => o.Amount > 0 && o.AmountPayment == o.Amount).Count();
                    rtn.AmountUnpaid = kv.Where(o => o.Amount > 0).Sum(o => o.Amount - o.AmountPayment);
                    rtn.QuantityUnpaid = kv.Where(o => o.Amount > 0 && o.AmountPayment < o.Amount).Count();
                    rtn.AmountOffset = kv.Where(o => o.Amount <= 0).Sum(o => o.Amount);
                    rtn.QuantityOffset = kv.Where(o => o.Amount <= 0).Count();
                    rtns.Results.Add(rtn);
                }
            }

            return rtns;
        }
    }
    #endregion

    #region Group
    [Api(Description = "结算统计Group")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementStatisticGroup : RestfulExecution<Settlement>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public AnalysisType GroupType { get; set; } = AnalysisType.Depart;//按时间,部门,类型分析

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Settlement>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime);
            sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.PayerKvid, o.PayerName, o.Type, o.Category, o.AmountPlan, o.Amount, o.AmountPayment, o.OperateTime, o.CreateTime });
            sqlExpress.Rows = null;
            var results = conn.Select(sqlExpress);

            var rtns = new StatisticResponse<Analysis>();
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            foreach (var item in results)
            {
                item.Summary = item.CreateTime.ToString("yyyy-MM");
            }
            if (GroupType == AnalysisType.Depart)
            {
                var group = results.GroupBy(o => o.OwnerName);
                foreach (var kv in group)
                {
                    var rtn = new Analysis();
                    rtn.Summary = kv.Key;
                    rtn.Amount = kv.Sum(o => o.Amount);
                    rtn.Quantity = kv.Count();
                    rtn.AmountPayment = kv.Sum(o => o.AmountPayment);
                    rtn.QuantityPayment = kv.Count(o => o.Amount == o.AmountPayment);
                    rtn.AmountPlan = kv.Sum(o => o.AmountPlan);
                    rtn.QuantityPlan = kv.Count();
                    rtn.AmountUnpaid = kv.Where(o => o.AmountPayment < o.Amount).Sum(o => o.Amount - o.AmountPayment);
                    rtn.QuantityUnpaid = kv.Count(o => o.AmountPayment < o.Amount);
                    rtns.Results.Add(rtn);
                }
            }
            else if (GroupType == AnalysisType.Payer)
            {
                var group = results.GroupBy(o => o.PayerName);
                foreach (var kv in group)
                {
                    var rtn = new Analysis();
                    rtn.Summary = kv.Key;
                    rtn.Amount = kv.Sum(o => o.Amount);
                    rtn.Quantity = kv.Count();
                    rtn.AmountPayment = kv.Sum(o => o.AmountPayment);
                    rtn.QuantityPayment = kv.Count(o => o.Amount == o.AmountPayment);
                    rtn.AmountPlan = kv.Sum(o => o.AmountPlan);
                    rtn.QuantityPlan = kv.Count();
                    rtn.AmountUnpaid = kv.Where(o => o.AmountPayment < o.Amount).Sum(o => o.Amount - o.AmountPayment);
                    rtn.QuantityUnpaid = kv.Count(o => o.AmountPayment < o.Amount);
                    rtns.Results.Add(rtn);
                }
            }
            else
            {
                var group = results.GroupBy(o => o.Summary);
                foreach (var kv in group)
                {
                    var rtn = new Analysis();
                    rtn.Summary = kv.Key;
                    rtn.Amount = kv.Sum(o => o.Amount);
                    rtn.Quantity = kv.Count();
                    rtn.AmountPayment = kv.Sum(o => o.AmountPayment);
                    rtn.QuantityPayment = kv.Count(o => o.Amount == o.AmountPayment);
                    rtn.AmountPlan = kv.Sum(o => o.AmountPlan);
                    rtn.QuantityPlan = kv.Count();
                    rtn.AmountUnpaid = kv.Where(o => o.AmountPayment < o.Amount).Sum(o => o.Amount - o.AmountPayment);
                    rtn.QuantityUnpaid = kv.Count(o => o.AmountPayment < o.Amount);
                    rtns.Results.Add(rtn);
                }
            }
            return rtns;
        }
    }
    #endregion
}
